"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var id_1 = require("../functions/id");
var isFunction_1 = require("../functions/isFunction");
var Reader = (function () {
    function Reader(fn) {
        if (!isFunction_1.default(fn))
            throw Error("Reader: Must wrap a function");
        this._value = fn;
    }
    Reader.of = function (v) {
        return new Reader(function () { return v; });
    };
    Reader.prototype.of = function (v) {
        return new Reader(function () { return v; });
    };
    Reader.prototype.ask = function (fn) {
        if (!fn)
            return new Reader(id_1.default);
        if (isFunction_1.default(fn))
            return new Reader(fn);
        throw Error("Reader: No argument or function required");
    };
    Reader.prototype.runWith = function (ctx) {
        return this._value(ctx);
    };
    Reader.prototype.ap = function (j) {
        if (!(j instanceof Reader))
            throw Error("Reader: Reader required");
        var runWith = this.getValue();
        return new Reader(function (e) {
            var fn = runWith(e);
            if (!isFunction_1.default(fn))
                throw Error("Reader: Wrapped function must return a function");
            return j.map(fn).runWith(e);
        });
    };
    Reader.prototype.getValue = function () {
        return this._value;
    };
    Reader.prototype.map = function (f) {
        if (!isFunction_1.default(f))
            throw Error("Reader: Expected a function");
        var runWith = this.getValue();
        return new Reader(function (x) { return f(runWith(x)); });
    };
    Reader.prototype.chain = function (f) {
        if (!isFunction_1.default(f))
            throw Error("Reader: Expected a function");
        var runWith = this.getValue();
        return new Reader(function (e) {
            var m = f(runWith(e));
            if (!(m instanceof Reader))
                throw Error("Reader: Function must return a Reader");
            return m.runWith(e);
        });
    };
    Reader.prototype.toString = function () {
        var v = this.getValue();
        return "Reader(" + v + ")";
    };
    return Reader;
}());
Reader.prototype["fantasy-land/map"] = Reader.prototype.map;
Reader.prototype["fantasy-land/chain"] = Reader.prototype.chain;
Reader.prototype["fantasy-land/of"] = Reader.prototype.of;
Reader.prototype["fantasy-land/ap"] = Reader.prototype.ap;
exports.default = Reader;
