"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var compose_1 = require("../functions/compose");
var isFunction_1 = require("../functions/isFunction");
var once_1 = require("../functions/once");
var Task = (function () {
    function Task(f) {
        if (isFunction_1.default(f))
            this._value = f;
        else
            throw Error("Task: Expected a Function");
    }
    Task.of = function (v) {
        return new Task(function (_, resolve) { return resolve(v); });
    };
    Task.rejected = function (v) {
        return new Task(function (reject, _) { return reject(v); });
    };
    Task.prototype.of = function (v) {
        return new Task(function (_, resolve) { return resolve(v); });
    };
    Task.prototype.rejected = function (v) {
        return new Task(function (reject, _) { return reject(v); });
    };
    Task.prototype.fork = function (reject, resolve) {
        if (!isFunction_1.default(resolve) || !isFunction_1.default(reject))
            throw Error("Task: Reject and Resolve need to be functions");
        var fn = this.getValue();
        fn(reject, resolve);
    };
    Task.prototype.toString = function () {
        var fork = this.getValue();
        return "Task(" + fork.name + ")";
    };
    Task.prototype.map = function (f) {
        if (!isFunction_1.default(f))
            throw Error("Task: Expected a function");
        var fork = this.getValue();
        return new Task(function (rej, res) {
            return fork(rej, compose_1.default(res, f));
        });
    };
    Task.prototype.getValue = function () {
        return this._value;
    };
    Task.prototype.ap = function (t) {
        if (!(t instanceof Task))
            throw Error("Task: type mismatch");
        var thisFork = this.getValue();
        var value;
        var fn;
        var gotValues = false;
        var gotFuction = false;
        var rejected = false;
        return new Task(function (rej, res) {
            var rejOnce = compose_1.default(function () {
                rejected = true;
            }, once_1.default(rej));
            var resolveBoth = function () {
                if (gotValues && gotFuction && !rejected) {
                    var exec = compose_1.default(res, fn);
                    exec.apply(null, value);
                }
            };
            thisFork(rejOnce, function () {
                var values = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    values[_i] = arguments[_i];
                }
                value = values;
                gotValues = true;
                resolveBoth();
            });
            t.fork(rejOnce, function (f) {
                if (!isFunction_1.default(f))
                    throw Error("Task: Wrapped value should be a function");
                fn = f;
                gotFuction = true;
                resolveBoth();
            });
        });
    };
    Task.prototype.concat = function (t) {
        if (!(t instanceof Task))
            throw Error("Task: type mismatch");
        var thisFork = this.getValue();
        var thatFork = t.getValue();
        return new Task(function (rej, res) {
            var rejected = false;
            var rejOnce = compose_1.default(function () {
                rejected = true;
            }, once_1.default(rej));
            var result1;
            var result2;
            var resolveBoth = function () {
                if (result1 && result2 && !rejected)
                    res.apply(null, tslib_1.__spreadArrays(result1, result2));
            };
            thisFork(rejOnce, function () {
                var values = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    values[_i] = arguments[_i];
                }
                result1 = values;
                resolveBoth();
            });
            thatFork(rejOnce, function () {
                var values = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    values[_i] = arguments[_i];
                }
                result2 = values;
                resolveBoth();
            });
        });
    };
    Task.prototype.chain = function (f) {
        if (!isFunction_1.default(f))
            throw Error("Task: Function required");
        var thisFork = this.getValue();
        return new Task(function (rej, res) {
            thisFork(rej, function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var t = f.call(null, args);
                if (!t.fork)
                    throw Error("Task: function must return another Task");
                t.fork(rej, res);
            });
        });
    };
    Task.prototype.toPromise = function () {
        var thisFork = this.getValue();
        return new Promise(function (res, rej) {
            thisFork(rej, res);
        });
    };
    return Task;
}());
Task.prototype["fantasy-land/map"] = Task.prototype.map;
Task.prototype["fantasy-land/concat"] = Task.prototype.concat;
Task.prototype["fantasy-land/chain"] = Task.prototype.chain;
Task.prototype["fantasy-land/of"] = Task.prototype.of;
Task.prototype["fantasy-land/ap"] = Task.prototype.ap;
exports.default = Task;
